package com.kmall.admin.controller;

import com.kmall.admin.entity.OrderGoodsEntity;
import com.kmall.admin.service.OrderGoodsService;
import com.kmall.common.utils.PageUtils;
import com.kmall.common.utils.Query;
import com.kmall.common.utils.R;
import com.kmall.common.utils.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;


/**
 * 
 * 
 * @author Scott
 * @email
 * @date 2017-08-13 10:41:09
 */
@RestController
@RequestMapping("ordergoods")
public class OrderGoodsController {
	@Autowired
	private OrderGoodsService orderGoodsService;
	
	/**
	 * 列表
	 */
	@RequestMapping("/list")
	@RequiresPermissions("ordergoods:list")
	public R list(@RequestParam Map<String, Object> params){
		//查询列表数据
        Query query = new Query(params);

		List<OrderGoodsEntity> orderGoodsList = orderGoodsService.queryList(query);
		int total = orderGoodsService.queryTotal(query);
		
		PageUtils pageUtil = new PageUtils(orderGoodsList, total, query.getLimit(), query.getPage());
		
		return R.ok().put("page", pageUtil);
	}
	
	
	/**
	 * 信息
	 */
	@RequestMapping("/info/{id}")
	@RequiresPermissions("ordergoods:info")
	public R info(@PathVariable("id") Integer id){
		OrderGoodsEntity orderGoods = orderGoodsService.queryObject(id);
		
		return R.ok().put("orderGoods", orderGoods);
	}
	
	/**
	 * 保存
	 */
	@RequestMapping("/save")
	@RequiresPermissions("ordergoods:save")
	public R save(@RequestBody OrderGoodsEntity orderGoods){
		orderGoodsService.save(orderGoods);
		
		return R.ok();
	}
	
	/**
	 * 修改
	 */
	@RequestMapping("/update")
	@RequiresPermissions("ordergoods:update")
	public R update(@RequestBody OrderGoodsEntity orderGoods){
		orderGoodsService.update(orderGoods);
		
		return R.ok();
	}
	
	/**
	 * 删除
	 */
	@RequestMapping("/delete")
	@RequiresPermissions("ordergoods:delete")
	public R delete(@RequestBody Integer[] ids){
		orderGoodsService.deleteBatch(ids);
		
		return R.ok();
	}

	/**
	 * 查看所有列表
	 */
	@RequestMapping("/queryAll")
	public R queryAll(@RequestParam Map<String, Object> params) {

		List<OrderGoodsEntity> list = orderGoodsService.queryList(params);

		return R.ok().put("list", list);
	}

	@RequestMapping("/queryListByOrderId")
//	@RequiresPermissions("ordergoods:queryListByOrderId")
	public R queryListByOrderId(@RequestParam Map<String, Object> params){
		//查询列表数据
		Integer orderId = Integer.parseInt((String) params.get("id"));
		List<OrderGoodsEntity> list = orderGoodsService.queryListByOrderId(orderId);
		if(params.get("promId") != null) {
			Integer promId = Integer.parseInt((String) params.get("promId"));
			for (OrderGoodsEntity orderGoodsEntity : list) {
				//是否属于当前推广渠道商品 0：否 1：是
				if (orderGoodsEntity.getIsPromGoods() == null) {
					orderGoodsEntity.setIsPromGoods(0);
				} else {
					if (orderGoodsEntity.getPromId().equals(promId)) {
						orderGoodsEntity.setIsPromGoods(1);
					} else {
						orderGoodsEntity.setIsPromGoods(0);
					}
				}
			}
		}

		PageUtils pageUtil = new PageUtils(list, 1, 5, 1);

		return R.ok().put("page", pageUtil);
	}
}
